@@ -15,7 +15,7 @@
  * @category   Zend
  * @package    Zend_Captcha
  * @subpackage Adapter
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -30,20 +30,20 @@ require_once 'Zend/Captcha/Word.php';
  * @category   Zend
  * @package    Zend_Captcha
  * @subpackage Adapter
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: $
+ * @version    $Id: Dumb.php 16971 2009-07-22 18:05:45Z mikaelkael $
 */
 class Zend_Captcha_Dumb extends Zend_Captcha_Word
 {
-	/**
-	 * Render the captcha
-	 *
-	 * @param  Zend_View $view
-	 * @param  mixed $element
-	 * @return string
-	 */
-	public function render(Zend_View $view, $element = null)
+    /**
+     * Render the captcha
+     *
+     * @param  Zend_View_Interface $view
+     * @param  mixed $element
+     * @return string
+     */
+    public function render(Zend_View_Interface $view = null, $element = null)
     {
         return 'Please type this word backwards: <b>'
              . strrev($this->getWord())
