@@ -14,16 +14,16 @@
  *
  * @category   Zend
  * @package    Zend_Config
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Config.php 11502 2008-09-23 20:16:32Z rob $
+ * @version    $Id: Config.php 16201 2009-06-21 18:51:15Z thomas $
  */
 
 
 /**
  * @category   Zend
  * @package    Zend_Config
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Config implements Countable, Iterator
@@ -56,6 +56,13 @@ class Zend_Config implements Countable, Iterator
      */
     protected $_data;
 
+    /**
+     * Used when unsetting values during iteration to ensure we do not skip
+     * the next element
+     *
+     * @var boolean
+     */
+    protected $_skipNextIteration;
 
     /**
      * Contains which config file sections were loaded. This is null
@@ -162,7 +169,7 @@ class Zend_Config implements Countable, Iterator
             throw new Zend_Config_Exception('Zend_Config is read only');
         }
     }
-
+    
     /**
      * Deep clone of this instance to ensure that nested Zend_Configs
      * are also cloned.
@@ -190,7 +197,8 @@ class Zend_Config implements Countable, Iterator
     public function toArray()
     {
         $array = array();
-        foreach ($this->_data as $key => $value) {
+        $data = $this->_data;
+        foreach ($data as $key => $value) {
             if ($value instanceof Zend_Config) {
                 $array[$key] = $value->toArray();
             } else {
@@ -223,6 +231,7 @@ class Zend_Config implements Countable, Iterator
         if ($this->_allowModifications) {
             unset($this->_data[$name]);
             $this->_count = count($this->_data);
+            $this->_skipNextIteration = true;
         } else {
             /** @see Zend_Config_Exception */
             require_once 'Zend/Config/Exception.php';
@@ -248,6 +257,7 @@ class Zend_Config implements Countable, Iterator
      */
     public function current()
     {
+        $this->_skipNextIteration = false;
         return current($this->_data);
     }
 
@@ -267,6 +277,10 @@ class Zend_Config implements Countable, Iterator
      */
     public function next()
     {
+        if ($this->_skipNextIteration) {
+            $this->_skipNextIteration = false;
+            return;
+        }
         next($this->_data);
         $this->_index++;
     }
@@ -277,6 +291,7 @@ class Zend_Config implements Countable, Iterator
      */
     public function rewind()
     {
+        $this->_skipNextIteration = false;
         reset($this->_data);
         $this->_index = 0;
     }
@@ -298,6 +313,9 @@ class Zend_Config implements Countable, Iterator
      */
     public function getSectionName()
     {
+        if(is_array($this->_loadedSection) && count($this->_loadedSection) == 1) {
+            $this->_loadedSection = $this->_loadedSection[0];
+        }
         return $this->_loadedSection;
     }
 
@@ -350,6 +368,11 @@ class Zend_Config implements Countable, Iterator
     public function setReadOnly()
     {
         $this->_allowModifications = false;
+        foreach ($this->_data as $key => $value) {
+            if ($value instanceof Zend_Config) {
+                $value->setReadOnly();
+            }
+        }
     }
     
     /**
@@ -362,6 +385,32 @@ class Zend_Config implements Countable, Iterator
         return !$this->_allowModifications;
     }
     
+    /**
+     * Get the current extends
+     *
+     * @return array
+     */
+    public function getExtends()
+    {
+        return $this->_extends;
+    }
+    
+    /**
+     * Set an extend for Zend_Config_Writer
+     *
+     * @param  string $extendingSection
+     * @param  string $extendedSection
+     * @return void
+     */
+    public function setExtend($extendingSection, $extendedSection = null)
+    {
+        if ($extendedSection === null && isset($this->_extends[$extendingSection])) {
+            unset($this->_extends[$extendingSection]);
+        } else if ($extendedSection !== null) {
+            $this->_extends[$extendingSection] = $extendedSection;
+        }
+    }
+    
     /**
      * Throws an exception if $extendingSection may not extend $extendedSection,
      * and tracks the section extension if it is valid.
