@@ -14,13 +14,11 @@
  *
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Http.php 18310 2009-09-19 17:50:02Z ralph $
  */
 
-/** Zend_Controller_Request_Exception */
-require_once 'Zend/Controller/Request/Exception.php';
-
 /** Zend_Controller_Request_Abstract */
 require_once 'Zend/Controller/Request/Abstract.php';
 
@@ -43,7 +41,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
      *
      */
     const SCHEME_HTTP  = 'http';
-    
+
     /**
      * Scheme for https
      *
@@ -86,6 +84,12 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
      */
     protected $_params = array();
 
+    /**
+     * Raw request body
+     * @var string|false
+     */
+    protected $_rawBody;
+
     /**
      * Alias keys for request parameters
      * @var array
@@ -237,9 +241,9 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
     /**
      * Set GET values
-     * 
-     * @param  string|array $spec 
-     * @param  null|mixed $value 
+     *
+     * @param  string|array $spec
+     * @param  null|mixed $value
      * @return Zend_Controller_Request_Http
      */
     public function setQuery($spec, $value = null)
@@ -279,9 +283,9 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
     /**
      * Set POST values
-     * 
-     * @param  string|array $spec 
-     * @param  null|mixed $value 
+     *
+     * @param  string|array $spec
+     * @param  null|mixed $value
      * @return Zend_Controller_Request_Http
      */
     public function setPost($spec, $value = null)
@@ -388,8 +392,21 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         if ($requestUri === null) {
             if (isset($_SERVER['HTTP_X_REWRITE_URL'])) { // check this first so IIS will catch
                 $requestUri = $_SERVER['HTTP_X_REWRITE_URL'];
+            } elseif (
+                // IIS7 with URL Rewrite: make sure we get the unencoded url (double slash problem)
+                isset($_SERVER['IIS_WasUrlRewritten'])
+                && $_SERVER['IIS_WasUrlRewritten'] == '1'
+                && isset($_SERVER['UNENCODED_URL'])
+                && $_SERVER['UNENCODED_URL'] != ''
+                ) {
+                $requestUri = $_SERVER['UNENCODED_URL'];
             } elseif (isset($_SERVER['REQUEST_URI'])) {
                 $requestUri = $_SERVER['REQUEST_URI'];
+                // Http proxy reqs setup request uri with scheme and host [and port] + the url path, only use url path
+                $schemeAndHttpHost = $this->getScheme() . '://' . $this->getHttpHost();
+                if (strpos($requestUri, $schemeAndHttpHost) === 0) {
+                    $requestUri = substr($requestUri, strlen($schemeAndHttpHost));
+                }
             } elseif (isset($_SERVER['ORIG_PATH_INFO'])) { // IIS 5.0, PHP as CGI
                 $requestUri = $_SERVER['ORIG_PATH_INFO'];
                 if (!empty($_SERVER['QUERY_STRING'])) {
@@ -456,19 +473,20 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         }
 
         if ($baseUrl === null) {
-            $filename = basename($_SERVER['SCRIPT_FILENAME']);
+            $filename = (isset($_SERVER['SCRIPT_FILENAME'])) ? basename($_SERVER['SCRIPT_FILENAME']) : '';
 
-            if (basename($_SERVER['SCRIPT_NAME']) === $filename) {
+            if (isset($_SERVER['SCRIPT_NAME']) && basename($_SERVER['SCRIPT_NAME']) === $filename) {
                 $baseUrl = $_SERVER['SCRIPT_NAME'];
-            } elseif (basename($_SERVER['PHP_SELF']) === $filename) {
+            } elseif (isset($_SERVER['PHP_SELF']) && basename($_SERVER['PHP_SELF']) === $filename) {
                 $baseUrl = $_SERVER['PHP_SELF'];
             } elseif (isset($_SERVER['ORIG_SCRIPT_NAME']) && basename($_SERVER['ORIG_SCRIPT_NAME']) === $filename) {
                 $baseUrl = $_SERVER['ORIG_SCRIPT_NAME']; // 1and1 shared hosting compatibility
             } else {
                 // Backtrack up the script_filename to find the portion matching
                 // php_self
-                $path    = $_SERVER['PHP_SELF'];
-                $segs    = explode('/', trim($_SERVER['SCRIPT_FILENAME'], '/'));
+                $path    = isset($_SERVER['PHP_SELF']) ? $_SERVER['PHP_SELF'] : '';
+                $file    = isset($_SERVER['SCRIPT_FILENAME']) ? $_SERVER['SCRIPT_FILENAME'] : '';
+                $segs    = explode('/', trim($file, '/'));
                 $segs    = array_reverse($segs);
                 $index   = 0;
                 $last    = count($segs);
@@ -495,7 +513,8 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
                 return $this;
             }
 
-            if (!strpos($requestUri, basename($baseUrl))) {
+            $basename = basename($baseUrl);
+            if (empty($basename) || !strpos($requestUri, $basename)) {
                 // no match whatsoever; set it blank
                 $this->_baseUrl = '';
                 return $this;
@@ -539,7 +558,9 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
     public function setBasePath($basePath = null)
     {
         if ($basePath === null) {
-            $filename = basename($_SERVER['SCRIPT_FILENAME']);
+            $filename = (isset($_SERVER['SCRIPT_FILENAME']))
+                      ? basename($_SERVER['SCRIPT_FILENAME'])
+                      : '';
 
             $baseUrl = $this->getBaseUrl();
             if (empty($baseUrl)) {
@@ -554,6 +575,10 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
             }
         }
 
+        if (substr(PHP_OS, 0, 3) === 'WIN') {
+            $basePath = str_replace('\\', '/', $basePath);
+        }
+
         $this->_basePath = rtrim($basePath, '/');
         return $this;
     }
@@ -627,8 +652,8 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
      * Set allowed parameter sources
      *
      * Can be empty array, or contain one or more of '_GET' or '_POST'.
-     * 
-     * @param  array $paramSoures 
+     *
+     * @param  array $paramSoures
      * @return Zend_Controller_Request_Http
      */
     public function setParamSources(array $paramSources = array())
@@ -639,7 +664,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
     /**
      * Get list of allowed parameter sources
-     * 
+     *
      * @return array
      */
     public function getParamSources()
@@ -704,12 +729,19 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
      */
     public function getParams()
     {
-        $return = $this->_params;
-        if (isset($_GET) && is_array($_GET)) {
-            $return += $_GET;
+        $return       = $this->_params;
+        $paramSources = $this->getParamSources();
+        if (in_array('_GET', $paramSources) 
+            && isset($_GET) 
+            && is_array($_GET)
+        ) { 
+            $return += $_GET; 
         }
-        if (isset($_POST) && is_array($_POST)) {
-            $return += $_POST;
+        if (in_array('_POST', $paramSources) 
+            && isset($_POST) 
+            && is_array($_POST)
+        ) { 
+            $return += $_POST; 
         }
         return $return;
     }
@@ -867,7 +899,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
         return false;
     }
-    
+
     /**
      * Is the request a Javascript XMLHttpRequest?
      *
@@ -882,14 +914,15 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
     /**
      * Is this a Flash request?
-     * 
+     *
      * @return bool
      */
     public function isFlashRequest()
     {
-        return ($this->getHeader('USER_AGENT') == 'Shockwave Flash');
+        $header = strtolower($this->getHeader('USER_AGENT'));
+        return (strstr($header, ' flash')) ? true : false;
     }
-    
+
     /**
      * Is https secure request
      *
@@ -899,7 +932,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
     {
         return ($this->getScheme() === self::SCHEME_HTTPS);
     }
-    
+
     /**
      * Return the raw body of the request, if present
      *
@@ -907,13 +940,16 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
      */
     public function getRawBody()
     {
-        $body = file_get_contents('php://input');
+        if (null === $this->_rawBody) {
+            $body = file_get_contents('php://input');
 
-        if (strlen(trim($body)) > 0) {
-            return $body;
+            if (strlen(trim($body)) > 0) {
+                $this->_rawBody = $body;
+            } else {
+                $this->_rawBody = false;
+            }
         }
-
-        return false;
+        return $this->_rawBody;
     }
 
     /**
@@ -949,7 +985,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
 
         return false;
     }
-    
+
     /**
      * Get the request URI scheme
      *
@@ -959,7 +995,7 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
     {
         return ($this->getServer('HTTPS') == 'on') ? self::SCHEME_HTTPS : self::SCHEME_HTTP;
     }
-    
+
     /**
      * Get the HTTP host.
      *
@@ -975,10 +1011,10 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
         if (!empty($host)) {
             return $host;
         }
-        
+
         $scheme = $this->getScheme();
         $name   = $this->getServer('SERVER_NAME');
-        $port   = $this->getServer('SERVER_PORT'); 
+        $port   = $this->getServer('SERVER_PORT');
 
         if (($scheme == self::SCHEME_HTTP && $port == 80) || ($scheme == self::SCHEME_HTTPS && $port == 443)) {
             return $name;
@@ -986,4 +1022,22 @@ class Zend_Controller_Request_Http extends Zend_Controller_Request_Abstract
             return $name . ':' . $port;
         }
     }
+
+    /**
+     * Get the client's IP addres
+     *
+     * @return string
+     */
+    public function getClientIp($checkProxy = true)
+    {
+        if ($checkProxy && $this->getServer('HTTP_CLIENT_IP') != null) {
+            $ip = $this->getServer('HTTP_CLIENT_IP');
+        } else if ($checkProxy && $this->getServer('HTTP_X_FORWARDED_FOR') != null) {
+            $ip = $this->getServer('HTTP_X_FORWARDED_FOR');
+        } else {
+            $ip = $this->getServer('REMOTE_ADDR');
+        }
+
+        return $ip;
+    }
 }
