@@ -15,8 +15,10 @@
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage YouTube
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: PlaylistListEntry.php 16971 2009-07-22 18:05:45Z mikaelkael $
  */
 
 /**
@@ -39,12 +41,23 @@ require_once 'Zend/Gdata/Extension/FeedLink.php';
  */
 require_once 'Zend/Gdata/YouTube/Extension/Description.php';
 
+/**
+ * @see Zend_Gdata_YouTube_Extension_PlaylistId
+ */
+require_once 'Zend/Gdata/YouTube/Extension/PlaylistId.php';
+
+/**
+ * @see Zend_Gdata_YouTube_Extension_CountHint
+ */
+require_once 'Zend/Gdata/YouTube/Extension/CountHint.php';
+
 /**
  * Represents the YouTube video playlist flavor of an Atom entry
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage YouTube
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Gdata_YouTube_PlaylistListEntry extends Zend_Gdata_Entry
@@ -62,10 +75,25 @@ class Zend_Gdata_YouTube_PlaylistListEntry extends Zend_Gdata_Entry
     /**
      * Description of this playlist
      *
-     * @var string
+     * @deprecated Deprecated as of version 2 of the YouTube API.
+     * @var Zend_Gdata_YouTube_Extension_Description
      */
     protected $_description = null;
 
+    /**
+     * Id of this playlist
+     *
+     * @var Zend_Gdata_YouTube_Extension_PlaylistId
+     */
+    protected $_playlistId = null;
+    
+    /**
+     * CountHint for this playlist.
+     *
+     * @var Zend_Gdata_YouTube_Extension_CountHint
+     */
+    protected $_countHint = null;
+
     /**
      * Creates a Playlist list entry, representing an individual playlist
      * in a list of playlists, usually associated with an individual user.
@@ -75,28 +103,32 @@ class Zend_Gdata_YouTube_PlaylistListEntry extends Zend_Gdata_Entry
      */
     public function __construct($element = null)
     {
-        foreach (Zend_Gdata_YouTube::$namespaces as $nsPrefix => $nsUri) {
-            $this->registerNamespace($nsPrefix, $nsUri); 
-        }
+        $this->registerAllNamespaces(Zend_Gdata_YouTube::$namespaces);
         parent::__construct($element);
     }
 
     /**
-     * Retrieves a DOMElement which corresponds to this element and all 
+     * Retrieves a DOMElement which corresponds to this element and all
      * child properties.  This is used to build an entry back into a DOM
      * and eventually XML text for sending to the server upon updates, or
-     * for application storage/persistence.   
+     * for application storage/persistence.
      *
      * @param DOMDocument $doc The DOMDocument used to construct DOMElements
-     * @return DOMElement The DOMElement representing this element and all 
-     * child properties. 
+     * @return DOMElement The DOMElement representing this element and all
+     * child properties.
      */
-    public function getDOM($doc = null)
+    public function getDOM($doc = null, $majorVersion = 1, $minorVersion = null)
     {
-        $element = parent::getDOM($doc);
+        $element = parent::getDOM($doc, $majorVersion, $minorVersion);
         if ($this->_description != null) {
             $element->appendChild($this->_description->getDOM($element->ownerDocument));
         }
+        if ($this->_countHint != null) {
+            $element->appendChild($this->_countHint->getDOM($element->ownerDocument));
+        }
+        if ($this->_playlistId != null) {
+            $element->appendChild($this->_playlistId->getDOM($element->ownerDocument));
+        }
         if ($this->_feedLink != null) {
             foreach ($this->_feedLink as $feedLink) {
                 $element->appendChild($feedLink->getDOM($element->ownerDocument));
@@ -120,6 +152,16 @@ class Zend_Gdata_YouTube_PlaylistListEntry extends Zend_Gdata_Entry
             $description->transferFromDOM($child);
             $this->_description = $description;
             break;
+        case $this->lookupNamespace('yt') . ':' . 'countHint':
+            $countHint = new Zend_Gdata_YouTube_Extension_CountHint();
+            $countHint->transferFromDOM($child);
+            $this->_countHint = $countHint;
+            break;
+        case $this->lookupNamespace('yt') . ':' . 'playlistId':
+            $playlistId = new Zend_Gdata_YouTube_Extension_PlaylistId();
+            $playlistId->transferFromDOM($child);
+            $this->_playlistId = $playlistId;
+            break;
         case $this->lookupNamespace('gd') . ':' . 'feedLink':
             $feedLink = new Zend_Gdata_Extension_FeedLink();
             $feedLink->transferFromDOM($child);
@@ -134,27 +176,76 @@ class Zend_Gdata_YouTube_PlaylistListEntry extends Zend_Gdata_Entry
     /**
      * Sets the description relating to the playlist.
      *
+     * @deprecated Deprecated as of version 2 of the YouTube API.
      * @param Zend_Gdata_YouTube_Extension_Description $description The description relating to the video
      * @return Zend_Gdata_YouTube_PlaylistListEntry Provides a fluent interface
-     */ 
-    public function setDescription($description = null) 
+     */
+    public function setDescription($description = null)
     {
-        $this->_description = $description;
+        if ($this->getMajorProtocolVersion() >= 2) {
+            $this->setSummary($description);
+        } else {
+            $this->_description = $description;
+        }
         return $this;
-    } 
+    }
 
     /**
      * Returns the description relating to the video.
      *
-     * @return Zend_Gdata_YouTube_Extension_Description  The description relating to the video 
+     * @return Zend_Gdata_YouTube_Extension_Description  The description 
+     *         relating to the video
      */
     public function getDescription()
     {
-        return $this->_description;
+        if ($this->getMajorProtocolVersion() >= 2) {
+            return $this->getSummary();
+        } else {
+            return $this->_description;
+        }
     }
 
     /**
-     * Sets the array of embedded feeds related to the video
+     * Returns the countHint relating to the playlist.
+     *
+     * The countHint is the number of videos on a playlist.
+     * 
+     * @throws Zend_Gdata_App_VersionException
+     * @return Zend_Gdata_YouTube_Extension_CountHint  The count of videos on
+     *         a playlist.
+     */
+    public function getCountHint()
+    {
+        if (($this->getMajorProtocolVersion() == null) ||
+            ($this->getMajorProtocolVersion() == 1)) {
+            require_once 'Zend/Gdata/App/VersionException.php';
+            throw new Zend_Gdata_App_VersionException('The yt:countHint ' . 
+                'element is not supported in versions earlier than 2.');
+        } else {
+            return $this->_countHint;
+        }
+    }
+
+    /**
+     * Returns the Id relating to the playlist.
+     * 
+     * @throws Zend_Gdata_App_VersionException
+     * @return Zend_Gdata_YouTube_Extension_PlaylistId  The id of this playlist.
+     */
+    public function getPlaylistId()
+    {
+        if (($this->getMajorProtocolVersion() == null) ||
+            ($this->getMajorProtocolVersion() == 1)) {
+            require_once 'Zend/Gdata/App/VersionException.php';
+            throw new Zend_Gdata_App_VersionException('The yt:playlistId ' . 
+                'element is not supported in versions earlier than 2.');
+        } else {
+            return $this->_playlistId;
+        }
+    }
+
+    /**
+     * Sets the array of embedded feeds related to the playlist
      *
      * @param array $feedLink The array of embedded feeds relating to the video
      * @return Zend_Gdata_YouTube_PlaylistListEntry Provides a fluent interface
@@ -199,7 +290,11 @@ class Zend_Gdata_YouTube_PlaylistListEntry extends Zend_Gdata_Entry
      */
     public function getPlaylistVideoFeedUrl()
     {
-        return $this->getFeedLink(Zend_Gdata_YouTube::PLAYLIST_REL)->href;
+        if ($this->getMajorProtocolVersion() >= 2) {
+            return $this->getContent()->getSrc();
+        } else {
+            return $this->getFeedLink(Zend_Gdata_YouTube::PLAYLIST_REL)->href;
+        }
     }
 
 }
