@@ -16,12 +16,18 @@
  * @category   Zend
  * @package    Zend_Http
  * @subpackage Client_Adapter
- * @version    $Id: Socket.php 11907 2008-10-12 17:20:19Z alexander $
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id: Socket.php 17124 2009-07-26 09:46:42Z shahar $
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
+/**
+ * @see Zend_Uri_Http
+ */
 require_once 'Zend/Uri/Http.php';
+/**
+ * @see Zend_Http_Client_Adapter_Interface
+ */
 require_once 'Zend/Http/Client/Adapter/Interface.php';
 
 /**
@@ -31,7 +37,7 @@ require_once 'Zend/Http/Client/Adapter/Interface.php';
  * @category   Zend
  * @package    Zend_Http
  * @subpackage Client_Adapter
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interface
@@ -69,6 +75,13 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
      */
     protected $method = null;
 
+    /**
+     * Stream context
+     *
+     * @var resource
+     */
+    protected $_context = null;
+
     /**
      * Adapter constructor, currently empty. Config is set using setConfig()
      *
@@ -80,14 +93,18 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
     /**
      * Set the configuration array for the adapter
      *
-     * @param array $config
+     * @param Zend_Config | array $config
      */
     public function setConfig($config = array())
     {
-        if (! is_array($config)) {
+        if ($config instanceof Zend_Config) {
+            $config = $config->toArray();
+
+        } elseif (! is_array($config)) {
             require_once 'Zend/Http/Client/Adapter/Exception.php';
             throw new Zend_Http_Client_Adapter_Exception(
-                '$config expects an array, ' . gettype($config) . ' recieved.');
+                'Array or Zend_Config object expected, got ' . gettype($config)
+            );
         }
 
         foreach ($config as $k => $v) {
@@ -95,13 +112,60 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
         }
     }
 
+    /**
+     * Set the stream context for the TCP connection to the server
+     *
+     * Can accept either a pre-existing stream context resource, or an array
+     * of stream options, similar to the options array passed to the
+     * stream_context_create() PHP function. In such case a new stream context
+     * will be created using the passed options.
+     *
+     * @since  Zend Framework 1.9
+     *
+     * @param  mixed $context Stream context or array of context options
+     * @return Zend_Http_Client_Adapter_Socket
+     */
+    public function setStreamContext($context)
+    {
+        if (is_resource($context) && get_resource_type($context) == 'stream-context') {
+            $this->_context = $context;
+
+        } elseif (is_array($context)) {
+            $this->_context = stream_context_create($context);
+
+        } else {
+            // Invalid parameter
+            require_once 'Zend/Http/Client/Adapter/Exception.php';
+            throw new Zend_Http_Client_Adapter_Exception(
+                "Expecting either a stream context resource or array, got " . gettype($context)
+            );
+        }
+
+        return $this;
+    }
+
+    /**
+     * Get the stream context for the TCP connection to the server.
+     *
+     * If no stream context is set, will create a default one.
+     *
+     * @return resource
+     */
+    public function getStreamContext()
+    {
+        if (! $this->_context) {
+            $this->_context = stream_context_create();
+        }
+
+        return $this->_context;
+    }
+
     /**
      * Connect to the remote server
      *
      * @param string  $host
      * @param int     $port
      * @param boolean $secure
-     * @param int     $timeout
      */
     public function connect($host, $port = 80, $secure = false)
     {
@@ -115,7 +179,7 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
 
         // Now, if we are not connected, connect
         if (! is_resource($this->socket) || ! $this->config['keepalive']) {
-            $context = stream_context_create();
+            $context = $this->getStreamContext();
             if ($secure) {
                 if ($this->config['sslcert'] !== null) {
                     if (! stream_context_set_option($context, 'ssl', 'local_cert',
@@ -135,13 +199,14 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
 
             $flags = STREAM_CLIENT_CONNECT;
             if ($this->config['persistent']) $flags |= STREAM_CLIENT_PERSISTENT;
-            
+
             $this->socket = @stream_socket_client($host . ':' . $port,
                                                   $errno,
                                                   $errstr,
                                                   (int) $this->config['timeout'],
                                                   $flags,
                                                   $context);
+
             if (! $this->socket) {
                 $this->close();
                 require_once 'Zend/Http/Client/Adapter/Exception.php';
@@ -219,6 +284,7 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
         // First, read headers only
         $response = '';
         $gotStatus = false;
+
         while (($line = @fgets($this->socket)) !== false) {
             $gotStatus = $gotStatus || (strpos($line, 'HTTP') !== false);
             if ($gotStatus) {
@@ -227,82 +293,121 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
             }
         }
 
+        $this->_checkSocketReadTimeout();
+
         $statusCode = Zend_Http_Response::extractCode($response);
-         
+
         // Handle 100 and 101 responses internally by restarting the read again
         if ($statusCode == 100 || $statusCode == 101) return $this->read();
 
+        // Check headers to see what kind of connection / transfer encoding we have
+        $headers = Zend_Http_Response::extractHeaders($response);
+
         /**
          * Responses to HEAD requests and 204 or 304 responses are not expected
          * to have a body - stop reading here
          */
-        if ($statusCode == 304 || $statusCode == 204 || 
-            $this->method == Zend_Http_Client::HEAD) return $response;
+        if ($statusCode == 304 || $statusCode == 204 ||
+            $this->method == Zend_Http_Client::HEAD) {
 
-        // Check headers to see what kind of connection / transfer encoding we have
-        $headers = Zend_Http_Response::extractHeaders($response);
+            // Close the connection if requested to do so by the server
+            if (isset($headers['connection']) && $headers['connection'] == 'close') {
+                $this->close();
+            }
+            return $response;
+        }
 
         // If we got a 'transfer-encoding: chunked' header
         if (isset($headers['transfer-encoding'])) {
-            if ($headers['transfer-encoding'] == 'chunked') {
+
+            if (strtolower($headers['transfer-encoding']) == 'chunked') {
+
                 do {
-                    $chunk = '';
-                    $line = @fgets($this->socket);
-                    $chunk .= $line;
+                    $line  = @fgets($this->socket);
+                    $this->_checkSocketReadTimeout();
 
-                    $hexchunksize = ltrim(chop($line), '0');
-                    $hexchunksize = strlen($hexchunksize) ? strtolower($hexchunksize) : 0;
+                    $chunk = $line;
 
-                    $chunksize = hexdec(chop($line));
-                    if (dechex($chunksize) != $hexchunksize) {
-                        @fclose($this->socket);
+                    // Figure out the next chunk size
+                    $chunksize = trim($line);
+                    if (! ctype_xdigit($chunksize)) {
+                        $this->close();
                         require_once 'Zend/Http/Client/Adapter/Exception.php';
                         throw new Zend_Http_Client_Adapter_Exception('Invalid chunk size "' .
-                            $hexchunksize . '" unable to read chunked body');
+                            $chunksize . '" unable to read chunked body');
                     }
 
-                    $left_to_read = $chunksize;
-                    while ($left_to_read > 0) {
-                        $line = @fread($this->socket, $left_to_read);
-                        $chunk .= $line;
-                        $left_to_read -= strlen($line);
-                        
-                        // Break if the connection ended prematurely
-                        if (feof($this->socket)) break;
-                    }
+                    // Convert the hexadecimal value to plain integer
+                    $chunksize = hexdec($chunksize);
+
+                    // Read next chunk
+                    $read_to = ftell($this->socket) + $chunksize;
+
+                    do {
+                        $current_pos = ftell($this->socket);
+                        if ($current_pos >= $read_to) break;
+
+                        $line = @fread($this->socket, $read_to - $current_pos);
+                        if ($line === false || strlen($line) === 0) {
+                            $this->_checkSocketReadTimeout();
+                            break;
+                        } else {
+                            $chunk .= $line;
+                        }
+
+                    } while (! feof($this->socket));
 
                     $chunk .= @fgets($this->socket);
+                    $this->_checkSocketReadTimeout();
+
                     $response .= $chunk;
                 } while ($chunksize > 0);
-                
+
             } else {
+                $this->close();
                 throw new Zend_Http_Client_Adapter_Exception('Cannot handle "' .
                     $headers['transfer-encoding'] . '" transfer encoding');
             }
-            
+
         // Else, if we got the content-length header, read this number of bytes
         } elseif (isset($headers['content-length'])) {
-            $left_to_read = $headers['content-length'];
+
+            $current_pos = ftell($this->socket);
             $chunk = '';
-            while ($left_to_read > 0) {
-                $chunk = @fread($this->socket, $left_to_read);
-                $left_to_read -= strlen($chunk);
+
+            for ($read_to = $current_pos + $headers['content-length'];
+                 $read_to > $current_pos;
+                 $current_pos = ftell($this->socket)) {
+
+                $chunk = @fread($this->socket, $read_to - $current_pos);
+                if ($chunk === false || strlen($chunk) === 0) {
+                    $this->_checkSocketReadTimeout();
+                    break;
+                }
+
                 $response .= $chunk;
-                
+
                 // Break if the connection ended prematurely
                 if (feof($this->socket)) break;
             }
-                    
+
         // Fallback: just read the response until EOF
         } else {
-            while (($buff = @fread($this->socket, 8192)) !== false) {
-                $response .= $buff;
-                if (feof($this->socket)) break;
-            }
+
+            do {
+                $buff = @fread($this->socket, 8192);
+                if ($buff === false || strlen($buff) === 0) {
+                    $this->_checkSocketReadTimeout();
+                    break;
+                } else {
+                    $response .= $buff;
+                }
+
+            } while (feof($this->socket) === false);
 
             $this->close();
         }
-        
+
         // Close the connection if requested to do so by the server
         if (isset($headers['connection']) && $headers['connection'] == 'close') {
             $this->close();
@@ -323,8 +428,30 @@ class Zend_Http_Client_Adapter_Socket implements Zend_Http_Client_Adapter_Interf
     }
 
     /**
-     * Destructor: make sure the socket is disconnected 
-     * 
+     * Check if the socket has timed out - if so close connection and throw
+     * an exception
+     *
+     * @throws Zend_Http_Client_Adapter_Exception with READ_TIMEOUT code
+     */
+    protected function _checkSocketReadTimeout()
+    {
+        if ($this->socket) {
+            $info = stream_get_meta_data($this->socket);
+            $timedout = $info['timed_out'];
+            if ($timedout) {
+                $this->close();
+                require_once 'Zend/Http/Client/Adapter/Exception.php';
+                throw new Zend_Http_Client_Adapter_Exception(
+                    "Read timed out after {$this->config['timeout']} seconds",
+                    Zend_Http_Client_Adapter_Exception::READ_TIMEOUT
+                );
+            }
+        }
+    }
+
+    /**
+     * Destructor: make sure the socket is disconnected
+     *
      * If we are in persistent TCP mode, will not close the connection
      *
      */
