@@ -14,9 +14,9 @@
  *
  * @category  Zend
  * @package   Zend_Measure
- * @copyright Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd     New BSD License
- * @version   $Id: Number.php 9534 2008-05-26 20:01:56Z thomas $
+ * @version   $Id: Number.php 16220 2009-06-21 19:49:21Z thomas $
  */
 
 /**
@@ -33,7 +33,7 @@ require_once 'Zend/Locale.php';
  * @category   Zend
  * @package    Zend_Measure
  * @subpackage Zend_Measure_Number
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Measure_Number extends Zend_Measure_Abstract
@@ -76,7 +76,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
 
     /**
      * Definition of all roman signs
-     * 
+     *
      * @var array $_roman
      */
     private static $_roman = array(
@@ -109,7 +109,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
 
     /**
      * Convertion table for roman signs
-     * 
+     *
      * @var array $_romanconvert
      */
     private static $_romanconvert = array(
@@ -144,7 +144,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
      */
     public function __construct($value, $type, $locale = null)
     {
-        if (Zend_Locale::isLocale($type) !== false) {
+        if (($type !== null) and (Zend_Locale::isLocale($type, null, false))) {
             $locale = $type;
             $type = null;
         }
@@ -153,16 +153,16 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
             $locale = new Zend_Locale();
         }
 
-        if ($locale instanceof Zend_Locale) {
-            $locale = $locale->toString();
+        if (!Zend_Locale::isLocale($locale, true, false)) {
+            if (!Zend_Locale::isLocale($locale, true, false)) {
+                require_once 'Zend/Measure/Exception.php';
+                throw new Zend_Measure_Exception("Language (" . (string) $locale . ") is unknown");
+            }
+
+            $locale = new Zend_Locale($locale);
         }
 
-        if (Zend_Locale::isLocale($locale, true) === false) {
-            require_once 'Zend/Measure/Exception.php';
-            throw new Zend_Measure_Exception("Language ($locale) is unknown");
-        }
-
-        $this->_locale = $locale;
+        $this->_locale = (string) $locale;
 
         if ($type === null) {
             $type = $this->_units['STANDARD'];
@@ -275,7 +275,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
      * @param  string  $type  Type from which to convert to decimal
      * @return string
      */
-    private function toDecimal($input, $type)
+    private function _toDecimal($input, $type)
     {
         $value = '';
         // Convert base xx values
@@ -336,7 +336,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
                 $target = call_user_func(Zend_Locale_Math::$mod, $value, $base);
 
                 $newvalue = strtoupper(dechex($target)) . $newvalue;
-                
+
                 $value = call_user_func(Zend_Locale_Math::$sub, $value, $target, 0);
                 $value = call_user_func(Zend_Locale_Math::$div, $value, $base, 0);
 
@@ -346,7 +346,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
                     throw new Zend_Measure_Exception("Your value '$tempvalue' cannot be processed because it extends 200 digits");
                 }
             }
-            
+
             if ($newvalue === '') {
                 $newvalue = '0';
             }
@@ -397,7 +397,7 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
             throw new Zend_Measure_Exception('Unknown type of number:' . $type);
         }
 
-        $value = $this->toDecimal($this->getValue(-1), $this->getType(-1));
+        $value = $this->_toDecimal($this->getValue(-1), $this->getType(-1));
         $value = $this->_fromDecimal($value, $type);
 
         $this->_value = $value;
@@ -412,9 +412,9 @@ class Zend_Measure_Number extends Zend_Measure_Abstract
      * @param  integer $round (Optional) Precision to add, will always be 0
      * @return string
      */
-    public function convertTo($type, $round = 0)
+    public function convertTo($type, $round = 0, $locale = null)
     {
         $this->setType($type);
-        return $this->toString($round);
+        return $this->toString($round, $locale);
     }
 }
