@@ -12,15 +12,13 @@
  * obtain it through the world-wide-web, please send an email
  * to license@zend.com so we can send you a copy immediately.
  *
+ * @category   Zend
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Page.php 17532 2009-08-10 19:04:14Z alexander $
  */
 
-
-/** Zend_Pdf_Exception */
-require_once 'Zend/Pdf/Exception.php';
-
 /** Zend_Pdf_Resource_Font */
 require_once 'Zend/Pdf/Resource/Font.php';
 
@@ -48,11 +46,14 @@ require_once 'Zend/Pdf/Color/Rgb.php';
 /** Zend_Pdf_Color_Cmyk */
 require_once 'Zend/Pdf/Color/Cmyk.php';
 
+/** Zend_Pdf_Annotation */
+require_once 'Zend/Pdf/Annotation.php';
+
 /**
  * PDF Page
  *
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Pdf_Page
@@ -124,18 +125,18 @@ class Zend_Pdf_Page
 
 
     /**
-     * Reference to the object with page dictionary.
+     * Page dictionary (refers to an inderect Zend_Pdf_Element_Dictionary object).
      *
-     * @var Zend_Pdf_Element_Reference
+     * @var Zend_Pdf_Element_Reference|Zend_Pdf_Element_Object
      */
-    private $_pageDictionary;
+    protected $_pageDictionary;
 
     /**
      * PDF objects factory.
      *
      * @var Zend_Pdf_ElementFactory_Interface
      */
-    private $_objFactory = null;
+    protected $_objFactory = null;
 
     /**
      * Flag which signals, that page is created separately from any PDF document or
@@ -143,53 +144,53 @@ class Zend_Pdf_Page
      *
      * @var boolean
      */
-    private $_attached;
+    protected $_attached;
 
     /**
-     * Stream of the drawing instractions.
+     * Stream of the drawing instructions.
      *
      * @var string
      */
-    private $_contents = '';
+    protected $_contents = '';
 
     /**
      * Current style
      *
      * @var Zend_Pdf_Style
      */
-    private $_style = null;
+    protected $_style = null;
 
     /**
      * Counter for the "Save" operations
      *
      * @var integer
      */
-    private $_saveCount = 0;
+    protected $_saveCount = 0;
 
     /**
      * Safe Graphics State semafore
-     * 
-     * If it's false, than we can't be sure Graphics State is restored withing 
+     *
+     * If it's false, than we can't be sure Graphics State is restored withing
      * context of previous contents stream (ex. drawing coordinate system may be rotated).
      * We should encompass existing content with save/restore GS operators
-     * 
-     * @var boolean 
+     *
+     * @var boolean
      */
-    private $_safeGS;
+    protected $_safeGS;
 
     /**
      * Current font
      *
      * @var Zend_Pdf_Resource_Font
      */
-    private $_font = null;
+    protected $_font = null;
 
     /**
      * Current font size
      *
      * @var float
      */
-    private $_fontSize;
+    protected $_fontSize;
 
     /**
      * Object constructor.
@@ -251,7 +252,7 @@ class Zend_Pdf_Page
             $this->_objFactory = $param1->_objFactory;
             $this->_attached   = &$param1->_attached;
             $this->_safeGS     = false;
-            
+
             $this->_pageDictionary = $this->_objFactory->newObject(new Zend_Pdf_Element_Dictionary());
 
             foreach ($param1->_pageDictionary->getKeys() as $key) {
@@ -308,6 +309,7 @@ class Zend_Pdf_Page
                  * @todo support of user defined pagesize notations, like:
                  *       "210x297mm", "595x842", "8.5x11in", "612x792"
                  */
+                require_once 'Zend/Pdf/Exception.php';
                 throw new Zend_Pdf_Exception('Wrong pagesize notation.');
             }
             /**
@@ -323,6 +325,7 @@ class Zend_Pdf_Page
             $pageHeight = $param2;
 
         } else {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Unrecognized method signature, wrong number of arguments or wrong argument types.');
         }
 
@@ -346,6 +349,7 @@ class Zend_Pdf_Page
      */
     public function __clone()
     {
+        require_once 'Zend/Pdf/Exception.php';
         throw new Zend_Pdf_Exception('Cloning Zend_Pdf_Page object using \'clone\' keyword is not supported. Use \'new Zend_Pdf_Page($srcPage)\' syntax');
     }
 
@@ -356,7 +360,7 @@ class Zend_Pdf_Page
      * @param Zend_Pdf_Resource $resource
      * @return string
      */
-    private function _attachResource($type, Zend_Pdf_Resource $resource)
+    protected function _attachResource($type, Zend_Pdf_Resource $resource)
     {
         // Check that Resources dictionary contains appropriate resource set
         if ($this->_pageDictionary->Resources->$type === null) {
@@ -384,13 +388,13 @@ class Zend_Pdf_Page
 
         return $newResName;
     }
-    
+
     /**
      * Add procedureSet to the Page description
      *
      * @param string $procSetName
      */
-    private function _addProcSet($procSetName)
+    protected function _addProcSet($procSetName)
     {
         // Check that Resources dictionary contains ProcSet entry
         if ($this->_pageDictionary->Resources->ProcSet === null) {
@@ -413,6 +417,7 @@ class Zend_Pdf_Page
     /**
      * Retrive PDF file reference to the page
      *
+     * @internal
      * @return Zend_Pdf_Element_Dictionary
      */
     public function getPageDictionary()
@@ -430,6 +435,7 @@ class Zend_Pdf_Page
     public function flush()
     {
         if ($this->_saveCount != 0) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Saved graphics state is not restored');
         }
 
@@ -452,26 +458,26 @@ class Zend_Pdf_Page
         }
 
         if ((!$this->_safeGS)  &&  (count($this->_pageDictionary->Contents->items) != 0)) {
-        	/**
-        	 * Page already has some content which is not treated as safe.
-        	 * 
-        	 * Add save/restore GS operators
-        	 */
+            /**
+             * Page already has some content which is not treated as safe.
+             *
+             * Add save/restore GS operators
+             */
             $this->_addProcSet('PDF');
-        	
-        	$newContentsArray = new Zend_Pdf_Element_Array();
-        	$newContentsArray->items[] = $this->_objFactory->newStreamObject(" q\n");
-        	foreach ($this->_pageDictionary->Contents->items as $contentStream) {
-        		$newContentsArray->items[] = $contentStream;
-        	}
+
+            $newContentsArray = new Zend_Pdf_Element_Array();
+            $newContentsArray->items[] = $this->_objFactory->newStreamObject(" q\n");
+            foreach ($this->_pageDictionary->Contents->items as $contentStream) {
+                $newContentsArray->items[] = $contentStream;
+            }
             $newContentsArray->items[] = $this->_objFactory->newStreamObject(" Q\n");
 
-        	$this->_pageDictionary->touch();
-        	$this->_pageDictionary->Contents = $newContentsArray;
-        	
-        	$this->_safeGS = true;
+            $this->_pageDictionary->touch();
+            $this->_pageDictionary->Contents = $newContentsArray;
+
+            $this->_safeGS = true;
         }
-        
+
         $this->_pageDictionary->Contents->items[] =
                 $this->_objFactory->newStreamObject($this->_contents);
 
@@ -496,6 +502,7 @@ class Zend_Pdf_Page
         }
 
         if ($this->_attached) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Page is attached to one documen, but rendered in context of another.');
             /**
              * @todo Page cloning must be implemented here instead of exception.
@@ -508,7 +515,6 @@ class Zend_Pdf_Page
         } else {
             $objFactory->attach($this->_objFactory);
         }
-
     }
 
 
@@ -517,34 +523,43 @@ class Zend_Pdf_Page
      * Set fill color.
      *
      * @param Zend_Pdf_Color $color
+     * @return Zend_Pdf_Page
      */
     public function setFillColor(Zend_Pdf_Color $color)
     {
         $this->_addProcSet('PDF');
         $this->_contents .= $color->instructions(false);
+
+        return $this;
     }
 
     /**
      * Set line color.
      *
      * @param Zend_Pdf_Color $color
+     * @return Zend_Pdf_Page
      */
     public function setLineColor(Zend_Pdf_Color $color)
     {
         $this->_addProcSet('PDF');
         $this->_contents .= $color->instructions(true);
+
+        return $this;
     }
 
     /**
      * Set line width.
      *
      * @param float $width
+     * @return Zend_Pdf_Page
      */
     public function setLineWidth($width)
     {
         $this->_addProcSet('PDF');
         $widthObj = new Zend_Pdf_Element_Numeric($width);
         $this->_contents .= $widthObj->toString() . " w\n";
+
+        return $this;
     }
 
     /**
@@ -555,6 +570,7 @@ class Zend_Pdf_Page
      *
      * @param array $pattern
      * @param array $phase
+     * @return Zend_Pdf_Page
      */
     public function setLineDashingPattern($pattern, $phase = 0)
     {
@@ -575,6 +591,8 @@ class Zend_Pdf_Page
 
         $this->_contents .= $dashPattern->toString() . ' '
                          . $phaseEleemnt->toString() . " d\n";
+
+        return $this;
     }
 
     /**
@@ -582,6 +600,7 @@ class Zend_Pdf_Page
      *
      * @param Zend_Pdf_Resource_Font $font
      * @param float $fontSize
+     * @return Zend_Pdf_Page
      */
     public function setFont(Zend_Pdf_Resource_Font $font, $fontSize)
     {
@@ -594,12 +613,15 @@ class Zend_Pdf_Page
         $fontNameObj = new Zend_Pdf_Element_Name($fontName);
         $fontSizeObj = new Zend_Pdf_Element_Numeric($fontSize);
         $this->_contents .= $fontNameObj->toString() . ' ' . $fontSizeObj->toString() . " Tf\n";
+
+        return $this;
     }
 
     /**
      * Set the style to use for future drawing operations on this page
      *
      * @param Zend_Pdf_Style $style
+     * @return Zend_Pdf_Page
      */
     public function setStyle(Zend_Pdf_Style $style)
     {
@@ -611,6 +633,8 @@ class Zend_Pdf_Page
             $this->setFont($style->getFont(), $style->getFontSize());
         }
         $this->_contents .= $style->instructions($this->_pageDictionary->Resources);
+
+        return $this;
     }
 
     /**
@@ -626,16 +650,19 @@ class Zend_Pdf_Page
      * @param float $alpha
      * @param string $mode
      * @throws Zend_Pdf_Exception
+     * @return Zend_Pdf_Page
      */
     public function setAlpha($alpha, $mode = 'Normal')
     {
-    	if (!in_array($mode, array('Normal', 'Multiply', 'Screen', 'Overlay', 'Darken', 'Lighten', 'ColorDodge',
-    	                           'ColorBurn', 'HardLight', 'SoftLight', 'Difference', 'Exclusion'))) {
-    	   throw new Zend_Pdf_Exception('Unsupported transparency mode.');
-    	}
-    	if (!is_numeric($alpha)  ||  $alpha < 0  ||  $alpha > 1) {
-    		throw new Zend_Pdf_Exception('Alpha value must be numeric between 0 (transparent) and 1 (opaque).');
-    	}
+        if (!in_array($mode, array('Normal', 'Multiply', 'Screen', 'Overlay', 'Darken', 'Lighten', 'ColorDodge',
+                                   'ColorBurn', 'HardLight', 'SoftLight', 'Difference', 'Exclusion'))) {
+            require_once 'Zend/Pdf/Exception.php';
+            throw new Zend_Pdf_Exception('Unsupported transparency mode.');
+        }
+        if (!is_numeric($alpha)  ||  $alpha < 0  ||  $alpha > 1) {
+            require_once 'Zend/Pdf/Exception.php';
+            throw new Zend_Pdf_Exception('Alpha value must be numeric between 0 (transparent) and 1 (opaque).');
+        }
 
         $this->_addProcSet('Text');
         $this->_addProcSet('PDF');
@@ -666,6 +693,8 @@ class Zend_Pdf_Page
 
         $gStateNameObj = new Zend_Pdf_Element_Name($gStateName);
         $this->_contents .= $gStateNameObj->toString() . " gs\n";
+
+        return $this;
     }
 
 
@@ -683,23 +712,24 @@ class Zend_Pdf_Page
      * Extract resources attached to the page
      *
      * This method is not intended to be used in userland, but helps to optimize some document wide operations
-     * 
+     *
      * returns array of Zend_Pdf_Element_Dictionary objects
-     * 
-     * @internal 
+     *
+     * @internal
      * @return array
      */
     public function extractResources()
     {
         return $this->_pageDictionary->Resources;
     }
-    
+
     /**
      * Extract fonts attached to the page
      *
      * returns array of Zend_Pdf_Resource_Font_Extracted objects
-     * 
+     *
      * @return array
+     * @throws Zend_Pdf_Exception
      */
     public function extractFonts()
     {
@@ -708,7 +738,7 @@ class Zend_Pdf_Page
             // Return empty array
             return array();
         }
-        
+
         $fontResources = $this->_pageDictionary->Resources->Font;
 
         $fontResourcesUnique = array();
@@ -717,36 +747,38 @@ class Zend_Pdf_Page
 
             if (! ($fontDictionary instanceof Zend_Pdf_Element_Reference  ||
                    $fontDictionary instanceof Zend_Pdf_Element_Object) ) {
-                // Font dictionary has to be an indirect object or object reference
-                continue;
+                require_once 'Zend/Pdf/Exception.php';
+                throw new Zend_Pdf_Exception('Font dictionary has to be an indirect object or object reference.');
             }
 
-            $fontResourcesUnique[$fontDictionary->toString($this->_objFactory)] = $fontDictionary;
+            $fontResourcesUnique[spl_object_hash($fontDictionary->getObject())] = $fontDictionary;
         }
-        
+
         $fonts = array();
-        foreach ($fontResourcesUnique as $resourceReference => $fontDictionary) {
+        require_once 'Zend/Pdf/Exception.php';
+        foreach ($fontResourcesUnique as $resourceId => $fontDictionary) {
             try {
                 // Try to extract font
                 $extractedFont = new Zend_Pdf_Resource_Font_Extracted($fontDictionary);
 
-                $fonts[$resourceReference] = $extractedFont; 
+                $fonts[$resourceId] = $extractedFont;
             } catch (Zend_Pdf_Exception $e) {
                 if ($e->getMessage() != 'Unsupported font type.') {
                     throw $e;
                 }
             }
         }
-        
+
         return $fonts;
-    } 
+    }
 
     /**
      * Extract font attached to the page by specific font name
-     * 
+     *
      * $fontName should be specified in UTF-8 encoding
      *
      * @return Zend_Pdf_Resource_Font_Extracted|null
+     * @throws Zend_Pdf_Exception
      */
     public function extractFont($fontName)
     {
@@ -754,36 +786,47 @@ class Zend_Pdf_Page
             // Page doesn't have any font attached
             return null;
         }
-        
+
         $fontResources = $this->_pageDictionary->Resources->Font;
 
+        $fontResourcesUnique = array();
+
+        require_once 'Zend/Pdf/Exception.php';
         foreach ($fontResources->getKeys() as $fontResourceName) {
             $fontDictionary = $fontResources->$fontResourceName;
-            
+
             if (! ($fontDictionary instanceof Zend_Pdf_Element_Reference  ||
                    $fontDictionary instanceof Zend_Pdf_Element_Object) ) {
-                // Font dictionary has to be an indirect object or object reference
-                continue;
+                require_once 'Zend/Pdf/Exception.php';
+                throw new Zend_Pdf_Exception('Font dictionary has to be an indirect object or object reference.');
             }
-            
+
+            $resourceId = spl_object_hash($fontDictionary->getObject());
+            if (isset($fontResourcesUnique[$resourceId])) {
+                continue;
+            } else {
+                // Mark resource as processed
+                $fontResourcesUnique[$resourceId] = 1;
+            }
+
             if ($fontDictionary->BaseFont->value != $fontName) {
                 continue;
             }
-            
+
             try {
                 // Try to extract font
-                return new Zend_Pdf_Resource_Font_Extracted($fontDictionary); 
+                return new Zend_Pdf_Resource_Font_Extracted($fontDictionary);
             } catch (Zend_Pdf_Exception $e) {
                 if ($e->getMessage() != 'Unsupported font type.') {
                     throw $e;
                 }
-                
+
                 // Continue searhing font with specified name
             }
         }
-        
+
         return null;
-    } 
+    }
 
     /**
      * Get current font size
@@ -813,6 +856,7 @@ class Zend_Pdf_Page
      *
      * @todo check for the open paths
      * @throws Zend_Pdf_Exception    - if a save is performed with an open path
+     * @return Zend_Pdf_Page
      */
     public function saveGS()
     {
@@ -820,19 +864,25 @@ class Zend_Pdf_Page
 
         $this->_addProcSet('PDF');
         $this->_contents .= " q\n";
+
+        return $this;
     }
 
     /**
      * Restore the graphics state that was saved with the last call to saveGS().
      *
      * @throws Zend_Pdf_Exception   - if there is no previously saved state
+     * @return Zend_Pdf_Page
      */
     public function restoreGS()
     {
         if ($this->_saveCount-- <= 0) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Restoring graphics state which is not saved');
         }
         $this->_contents .= " Q\n";
+
+        return $this;
     }
 
 
@@ -844,12 +894,15 @@ class Zend_Pdf_Page
      * @param float $radius
      * @param float $startAngle
      * @param float $endAngle
+     * @return Zend_Pdf_Page
      */
     public function clipCircle($x, $y, $radius, $startAngle = null, $endAngle = null)
     {
         $this->clipEllipse($x - $radius, $y - $radius,
                            $x + $radius, $y + $radius,
                            $startAngle, $endAngle);
+
+        return $this;
     }
 
     /**
@@ -867,6 +920,7 @@ class Zend_Pdf_Page
      * @param float $y2
      * @param float $startAngle
      * @param float $endAngle
+     * @return Zend_Pdf_Page
      */
     public function clipEllipse($x1, $y1, $x2, $y2, $startAngle = null, $endAngle = null)
     {
@@ -938,6 +992,8 @@ class Zend_Pdf_Page
                          .    $xl->toString() . ' ' . $yUp->toString() . ' '
                          .      $xC->toString() . ' ' . $yUp->toString() . " c\n"
                          .  "h\nW\nn\n";
+
+        return $this;
     }
 
 
@@ -947,6 +1003,7 @@ class Zend_Pdf_Page
      * @param array $x  - array of float (the X co-ordinates of the vertices)
      * @param array $y  - array of float (the Y co-ordinates of the vertices)
      * @param integer $fillMethod
+     * @return Zend_Pdf_Page
      */
     public function clipPolygon($x, $y, $fillMethod = Zend_Pdf_Page::FILL_METHOD_NON_ZERO_WINDING)
     {
@@ -968,11 +1025,13 @@ class Zend_Pdf_Page
         $this->_contents .= $path;
 
         if ($fillMethod == Zend_Pdf_Page::FILL_METHOD_NON_ZERO_WINDING) {
-            $this->_contents .= " h\n W\n";
+            $this->_contents .= " h\n W\nn\n";
         } else {
             // Even-Odd fill method.
-            $this->_contents .= " h\n W*\n";
+            $this->_contents .= " h\n W*\nn\n";
         }
+
+        return $this;
     }
 
     /**
@@ -982,6 +1041,7 @@ class Zend_Pdf_Page
      * @param float $y1
      * @param float $x2
      * @param float $y2
+     * @return Zend_Pdf_Page
      */
     public function clipRectangle($x1, $y1, $x2, $y2)
     {
@@ -994,7 +1054,9 @@ class Zend_Pdf_Page
 
         $this->_contents .= $x1Obj->toString() . ' ' . $y1Obj->toString() . ' '
                          .      $widthObj->toString() . ' ' . $height2Obj->toString() . " re\n"
-                         .  " W\n";
+                         .  " W\nn\n";
+
+        return $this;
     }
 
     /**
@@ -1005,9 +1067,12 @@ class Zend_Pdf_Page
      * @param float $y1
      * @param float $x2
      * @param float $y2
+     * @return Zend_Pdf_Page
      */
     public function drawContentStream($cs, $x1, $y1, $x2, $y2)
     {
+        /** @todo implementation */
+        return $this;
     }
 
     /**
@@ -1032,12 +1097,15 @@ class Zend_Pdf_Page
      * @param mixed $param4
      * @param mixed $param5
      * @param mixed $param6
+     * @return Zend_Pdf_Page
      */
     public function  drawCircle($x, $y, $radius, $param4 = null, $param5 = null, $param6 = null)
     {
         $this->drawEllipse($x - $radius, $y - $radius,
                            $x + $radius, $y + $radius,
                            $param4, $param5, $param6);
+
+        return $this;
     }
 
     /**
@@ -1058,6 +1126,7 @@ class Zend_Pdf_Page
      * @param mixed $param5
      * @param mixed $param6
      * @param mixed $param7
+     * @return Zend_Pdf_Page
      */
     public function drawEllipse($x1, $y1, $x2, $y2, $param5 = null, $param6 = null, $param7 = null)
     {
@@ -1165,6 +1234,8 @@ class Zend_Pdf_Page
         if ($startAngle !== null) {
             $this->_contents .= "Q\n";
         }
+
+        return $this;
     }
 
     /**
@@ -1175,6 +1246,7 @@ class Zend_Pdf_Page
      * @param float $y1
      * @param float $x2
      * @param float $y2
+     * @return Zend_Pdf_Page
      */
     public function drawImage(Zend_Pdf_Resource_Image $image, $x1, $y1, $x2, $y2)
     {
@@ -1193,6 +1265,8 @@ class Zend_Pdf_Page
                          .  $widthObj->toString() . ' 0 0 ' . $heightObj->toString() . " 0 0 cm\n"
                          .  $imageNameObj->toString() . " Do\n"
                          .  "Q\n";
+
+        return $this;
     }
 
     /**
@@ -1201,9 +1275,12 @@ class Zend_Pdf_Page
      * @param Zend_Pdf_Element_LayoutBox $box
      * @param float $x
      * @param float $y
+     * @return Zend_Pdf_Page
      */
     public function drawLayoutBox($box, $x, $y)
     {
+        /** @todo implementation */
+        return $this;
     }
 
     /**
@@ -1213,6 +1290,7 @@ class Zend_Pdf_Page
      * @param float $y1
      * @param float $x2
      * @param float $y2
+     * @return Zend_Pdf_Page
      */
     public function drawLine($x1, $y1, $x2, $y2)
     {
@@ -1225,6 +1303,8 @@ class Zend_Pdf_Page
 
         $this->_contents .= $x1Obj->toString() . ' ' . $y1Obj->toString() . " m\n"
                          .  $x2Obj->toString() . ' ' . $y2Obj->toString() . " l\n S\n";
+
+        return $this;
     }
 
     /**
@@ -1239,6 +1319,7 @@ class Zend_Pdf_Page
      * @param array $y  - array of float (the Y co-ordinates of the vertices)
      * @param integer $fillType
      * @param integer $fillMethod
+     * @return Zend_Pdf_Page
      */
     public function drawPolygon($x, $y,
                                 $fillType = Zend_Pdf_Page::SHAPE_DRAW_FILL_AND_STROKE,
@@ -1282,6 +1363,8 @@ class Zend_Pdf_Page
                 $this->_contents .= " S\n";
                 break;
         }
+
+        return $this;
     }
 
     /**
@@ -1297,6 +1380,7 @@ class Zend_Pdf_Page
      * @param float $x2
      * @param float $y2
      * @param integer $fillType
+     * @return Zend_Pdf_Page
      */
     public function drawRectangle($x1, $y1, $x2, $y2, $fillType = Zend_Pdf_Page::SHAPE_DRAW_FILL_AND_STROKE)
     {
@@ -1321,6 +1405,8 @@ class Zend_Pdf_Page
                 $this->_contents .= " S\n";
                 break;
         }
+
+        return $this;
     }
 
     /**
@@ -1332,10 +1418,12 @@ class Zend_Pdf_Page
      * @param string $charEncoding (optional) Character encoding of source text.
      *   Defaults to current locale.
      * @throws Zend_Pdf_Exception
+     * @return Zend_Pdf_Page
      */
     public function drawText($text, $x, $y, $charEncoding = '')
     {
         if ($this->_font === null) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Font has not been set');
         }
 
@@ -1349,6 +1437,36 @@ class Zend_Pdf_Page
                          .  $xObj->toString() . ' ' . $yObj->toString() . " Td\n"
                          .  $textObj->toString() . " Tj\n"
                          .  "ET\n";
+
+        return $this;
+    }
+
+    /**
+     *
+     * @param Zend_Pdf_Annotation $annotation
+     * @return Zend_Pdf_Page
+     */
+    public function attachAnnotation(Zend_Pdf_Annotation $annotation)
+    {
+        $annotationDictionary = $annotation->getResource();
+        if (!$annotationDictionary instanceof Zend_Pdf_Element_Object  &&
+            !$annotationDictionary instanceof Zend_Pdf_Element_Reference) {
+            $annotationDictionary = $this->_objFactory->newObject($annotationDictionary);
+        }
+
+        if ($this->_pageDictionary->Annots === null) {
+            $this->_pageDictionary->touch();
+            $this->_pageDictionary->Annots = new Zend_Pdf_Element_Array();
+        } else {
+            $this->_pageDictionary->Annots->touch();
+        }
+
+        $this->_pageDictionary->Annots->items[] = $annotationDictionary;
+
+        $annotationDictionary->touch();
+        $annotationDictionary->P = $this->_pageDictionary;
+
+        return $this;
     }
 
     /**
@@ -1377,9 +1495,12 @@ class Zend_Pdf_Page
      * Close the path by drawing a straight line back to it's beginning.
      *
      * @throws Zend_Pdf_Exception    - if a path hasn't been started with pathMove()
+     * @return Zend_Pdf_Page
      */
     public function pathClose()
     {
+        /** @todo implementation */
+        return $this;
     }
 
     /**
@@ -1387,9 +1508,12 @@ class Zend_Pdf_Page
      *
      * @param float $x  - the X co-ordinate to move to
      * @param float $y  - the Y co-ordinate to move to
+     * @return Zend_Pdf_Page
      */
     public function pathLine($x, $y)
     {
+        /** @todo implementation */
+        return $this;
     }
 
     /**
@@ -1398,24 +1522,41 @@ class Zend_Pdf_Page
      *
      * @param float $x  - the X co-ordinate to move to
      * @param float $y  - the Y co-ordinate to move to
+     * @return Zend_Pdf_Page
      */
     public function pathMove($x, $y)
     {
+        /** @todo implementation */
+        return $this;
     }
 
     /**
-     * Write raw PDF commands to the page.
+     * Writes the raw data to the page's content stream.
+     *
+     * Be sure to consult the PDF reference to ensure your syntax is correct. No
+     * attempt is made to ensure the validity of the stream data.
      *
      * @param string $data
+     * @param string $procSet (optional) Name of ProcSet to add.
+     * @return Zend_Pdf_Page
      */
-    public function rawWrite($data)
+    public function rawWrite($data, $procSet = null)
     {
+        if (! empty($procSet)) {
+            $this->_addProcSet($procSet);
+        }
+        $this->_contents .= $data;
+
+        return $this;
     }
 
     /**
      * Rotate the page.
      *
-     * @param float $angle
+     * @param float $x  - the X co-ordinate of rotation point
+     * @param float $y  - the Y co-ordinate of rotation point
+     * @param float $angle - rotation angle
+     * @return Zend_Pdf_Page
      */
     public function rotate($x, $y, $angle)
     {
@@ -1432,11 +1573,74 @@ class Zend_Pdf_Page
 
         $this->_addProcSet('PDF');
         $this->_contents .= '1 0 0 1 ' . $xObj->toString() . ' ' . $yObj->toString() . " cm\n"
-                         . $cos->toString() . ' ' . $sin->toString()
-                         . ' ' . $mSin->toString() . ' ' . $cos->toString() . " 0 0 cm\n"
-                         .'1 0 0 1 ' . $mXObj->toString() . ' ' . $mYObj->toString() . " cm\n";
+                         .  $cos->toString() . ' ' . $sin->toString() . ' ' . $mSin->toString() . ' ' . $cos->toString() . " 0 0 cm\n"
+                         .  '1 0 0 1 ' . $mXObj->toString() . ' ' . $mYObj->toString() . " cm\n";
+
+        return $this;
     }
 
+    /**
+     * Scale coordination system.
+     *
+     * @param float $xScale - X dimention scale factor
+     * @param float $yScale - Y dimention scale factor
+     * @return Zend_Pdf_Page
+     */
+    public function scale($xScale, $yScale)
+    {
+        $xScaleObj = new Zend_Pdf_Element_Numeric($xScale);
+        $yScaleObj = new Zend_Pdf_Element_Numeric($yScale);
 
+        $this->_addProcSet('PDF');
+        $this->_contents .= $xScaleObj->toString() . ' 0 0 ' . $yScaleObj->toString() . " 0 0 cm\n";
+
+        return $this;
+    }
+
+    /**
+     * Translate coordination system.
+     *
+     * @param float $xShift - X coordinate shift
+     * @param float $yShift - Y coordinate shift
+     * @return Zend_Pdf_Page
+     */
+    public function translate($xShift, $yShift)
+    {
+        $xShiftObj = new Zend_Pdf_Element_Numeric($xShift);
+        $yShiftObj = new Zend_Pdf_Element_Numeric($yShift);
+
+        $this->_addProcSet('PDF');
+        $this->_contents .= '1 0 0 1 ' . $xShiftObj->toString() . ' ' . $yShiftObj->toString() . " cm\n";
+
+        return $this;
+    }
+
+    /**
+     * Translate coordination system.
+     *
+     * @param float $x  - the X co-ordinate of axis skew point
+     * @param float $y  - the Y co-ordinate of axis skew point
+     * @param float $xAngle - X axis skew angle
+     * @param float $yAngle - Y axis skew angle
+     * @return Zend_Pdf_Page
+     */
+    public function skew($x, $y, $xAngle, $yAngle)
+    {
+        $tanXObj = new Zend_Pdf_Element_Numeric(tan($xAngle));
+        $tanYObj = new Zend_Pdf_Element_Numeric(-tan($yAngle));
+
+        $xObj = new Zend_Pdf_Element_Numeric($x);
+        $yObj = new Zend_Pdf_Element_Numeric($y);
+
+        $mXObj = new Zend_Pdf_Element_Numeric(-$x);
+        $mYObj = new Zend_Pdf_Element_Numeric(-$y);
+
+        $this->_addProcSet('PDF');
+        $this->_contents .= '1 0 0 1 ' . $xObj->toString() . ' ' . $yObj->toString() . " cm\n"
+                         .  '1 ' . $tanXObj->toString() . ' ' . $tanYObj->toString() . " 1 0 0 cm\n"
+                         .  '1 0 0 1 ' . $mXObj->toString() . ' ' . $mYObj->toString() . " cm\n";
+
+        return $this;
+    }
 }
 
