@@ -15,8 +15,9 @@
  * @category   Zend
  * @package    Zend_Search_Lucene
  * @subpackage Index
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Writer.php 16541 2009-07-07 06:59:03Z bkarwin $
  */
 
 
@@ -38,7 +39,7 @@ require_once 'Zend/Search/Lucene/LockManager.php';
  * @category   Zend
  * @package    Zend_Search_Lucene
  * @subpackage Index
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Search_Lucene_Index_Writer
@@ -189,11 +190,8 @@ class Zend_Search_Lucene_Index_Writer
             $segmentsFile = $directory->createFile('segments');
             $segmentsFile->writeInt((int)0xFFFFFFFF);
 
-            // write version (is initialized by current time
-            // $segmentsFile->writeLong((int)microtime(true));
-            $version = microtime(true);
-            $segmentsFile->writeInt((int)($version/((double)0xFFFFFFFF + 1)));
-            $segmentsFile->writeInt((int)($version & 0xFFFFFFFF));
+            // write version (initialized by current time)
+            $segmentsFile->writeLong(round(microtime(true)));
 
             // write name counter
             $segmentsFile->writeInt($nameCount);
@@ -214,11 +212,8 @@ class Zend_Search_Lucene_Index_Writer
             $segmentsFile = $directory->createFile(Zend_Search_Lucene::getSegmentFileName($generation));
             $segmentsFile->writeInt((int)0xFFFFFFFD);
 
-            // write version (is initialized by current time
-            // $segmentsFile->writeLong((int)microtime(true));
-            $version = microtime(true);
-            $segmentsFile->writeInt((int)($version/((double)0xFFFFFFFF + 1)));
-            $segmentsFile->writeInt((int)($version & 0xFFFFFFFF));
+            // write version (initialized by current time)
+            $segmentsFile->writeLong(round(microtime(true)));
 
             // write name counter
             $segmentsFile->writeInt($nameCount);
@@ -435,10 +430,10 @@ class Zend_Search_Lucene_Index_Writer
 
         try {
             // Write format marker
-            if ($this->_targetFormatVersion == Zend_Search_lucene::FORMAT_2_1) {
-            	$newSegmentFile->writeInt((int)0xFFFFFFFD);
-            } else if ($this->_targetFormatVersion == Zend_Search_lucene::FORMAT_2_3) {
-            	$newSegmentFile->writeInt((int)0xFFFFFFFC);
+            if ($this->_targetFormatVersion == Zend_Search_Lucene::FORMAT_2_1) {
+                $newSegmentFile->writeInt((int)0xFFFFFFFD);
+            } else if ($this->_targetFormatVersion == Zend_Search_Lucene::FORMAT_2_3) {
+                $newSegmentFile->writeInt((int)0xFFFFFFFC);
             }
 
             // Read src file format identifier
@@ -453,16 +448,9 @@ class Zend_Search_Lucene_Index_Writer
                 throw new Zend_Search_Lucene_Exception('Unsupported segments file format');
             }
 
-            // $version = $segmentsFile->readLong() + $this->_versionUpdate;
-            // Process version on 32-bit platforms
-            $versionHigh = $segmentsFile->readInt();
-            $versionLow  = $segmentsFile->readInt();
-            $version = $versionHigh * ((double)0xFFFFFFFF + 1) +
-                       (($versionLow < 0)? (double)0xFFFFFFFF - (-1 - $versionLow) : $versionLow);
-            $version += $this->_versionUpdate;
+            $version = $segmentsFile->readLong() + $this->_versionUpdate;
             $this->_versionUpdate = 0;
-            $newSegmentFile->writeInt((int)($version/((double)0xFFFFFFFF + 1)));
-            $newSegmentFile->writeInt((int)($version & 0xFFFFFFFF));
+            $newSegmentFile->writeLong($version);
 
             // Write segment name counter
             $newSegmentFile->writeInt($segmentsFile->readInt());
@@ -482,21 +470,18 @@ class Zend_Search_Lucene_Index_Writer
 
                 if ($srcFormat == Zend_Search_Lucene::FORMAT_PRE_2_1) {
                     // pre-2.1 index format
-                    $delGenHigh        = 0;
-                    $delGenLow         = 0;
+                    $delGen            = 0;
                     $hasSingleNormFile = false;
                     $numField          = (int)0xFFFFFFFF;
                     $isCompoundByte    = 0;
                     $docStoreOptions   = null;
                 } else {
-                    //$delGen          = $segmentsFile->readLong();
-                    $delGenHigh        = $segmentsFile->readInt();
-                    $delGenLow         = $segmentsFile->readInt();
+                    $delGen = $segmentsFile->readLong();
 
                     if ($srcFormat == Zend_Search_Lucene::FORMAT_2_3) {
-                    	$docStoreOffset = $segmentsFile->readInt();
+                        $docStoreOffset = $segmentsFile->readInt();
 
-                        if ($docStoreOffset != -1) {
+                        if ($docStoreOffset != (int)0xFFFFFFFF) {
                             $docStoreSegment        = $segmentsFile->readString();
                             $docStoreIsCompoundFile = $segmentsFile->readByte();
 
@@ -507,7 +492,7 @@ class Zend_Search_Lucene_Index_Writer
                             $docStoreOptions = null;
                         }
                     } else {
-                    	$docStoreOptions = null;
+                        $docStoreOptions = null;
                     }
 
                     $hasSingleNormFile = $segmentsFile->readByte();
@@ -525,8 +510,6 @@ class Zend_Search_Lucene_Index_Writer
                 if (!in_array($segName, $this->_segmentsToDelete)) {
                     // Load segment if necessary
                     if (!isset($this->_segmentInfos[$segName])) {
-                        $delGen = $delGenHigh * ((double)0xFFFFFFFF + 1) +
-                                     (($delGenLow < 0)? (double)0xFFFFFFFF - (-1 - $delGenLow) : $delGenLow);
                         if ($isCompoundByte == 0xFF) {
                             // The segment is not a compound file
                             $isCompound = false;
@@ -549,28 +532,20 @@ class Zend_Search_Lucene_Index_Writer
                     } else {
                         // Retrieve actual deletions file generation number
                         $delGen = $this->_segmentInfos[$segName]->getDelGen();
-
-                        if ($delGen >= 0) {
-                            $delGenHigh = (int)($delGen/((double)0xFFFFFFFF + 1));
-                            $delGenLow  =(int)($delGen & 0xFFFFFFFF);
-                        } else {
-                            $delGenHigh = $delGenLow = (int)0xFFFFFFFF;
-                        }
                     }
 
                     $newSegmentFile->writeString($segName);
                     $newSegmentFile->writeInt($segSize);
-                    $newSegmentFile->writeInt($delGenHigh);
-                    $newSegmentFile->writeInt($delGenLow);
+                    $newSegmentFile->writeLong($delGen);
                     if ($this->_targetFormatVersion == Zend_Search_Lucene::FORMAT_2_3) {
-                    	if ($docStoreOptions !== null) {
-                    		$newSegmentFile->writeInt($docStoreOffset);
-                    		$newSegmentFile->writeString($docStoreSegment);
-                    		$newSegmentFile->writeByte($docStoreIsCompoundFile);
-                    	} else {
-                    		// Set DocStoreOffset to -1
-                    		$newSegmentFile->writeInt((int)0xFFFFFFFF);
-                    	}
+                        if ($docStoreOptions !== null) {
+                            $newSegmentFile->writeInt($docStoreOffset);
+                            $newSegmentFile->writeString($docStoreSegment);
+                            $newSegmentFile->writeByte($docStoreIsCompoundFile);
+                        } else {
+                            // Set DocStoreOffset to -1
+                            $newSegmentFile->writeInt((int)0xFFFFFFFF);
+                        }
                     } else if ($docStoreOptions !== null) {
                         // Release index write lock
                         Zend_Search_Lucene_LockManager::releaseWriteLock($this->_directory);
@@ -734,11 +709,11 @@ class Zend_Search_Lucene_Index_Writer
 
             foreach ($filesToDelete as $file) {
                 try {
-                	/** Skip shared docstore segments deleting */
+                    /** Skip shared docstore segments deleting */
                     /** @todo Process '.cfx' files to check if them are already unused */
-                	if (substr($file, strlen($file)-4) != '.cfx') {
+                    if (substr($file, strlen($file)-4) != '.cfx') {
                         $this->_directory->deleteFile($file);
-                	}
+                    }
                 } catch (Zend_Search_Lucene_Exception $e) {
                     if (strpos($e->getMessage(), 'Can\'t delete file') === false) {
                         // That's not "file is under processing or already deleted" exception
