@@ -14,13 +14,12 @@
  *
  * @category   Zend
  * @package    Zend_Soap
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Client
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Client.php 16971 2009-07-22 18:05:45Z mikaelkael $
  */
 
-/** Zend_Soap_Client_Exception */
-require_once 'Zend/Soap/Client/Exception.php';
-
 /** Zend_Soap_Server */
 require_once 'Zend/Soap/Server.php';
 
@@ -30,13 +29,13 @@ require_once 'Zend/Soap/Client/Local.php';
 /** Zend_Soap_Client_Common */
 require_once 'Zend/Soap/Client/Common.php';
 
-
 /**
  * Zend_Soap_Client
  *
  * @category   Zend
  * @package    Zend_Soap
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Client
+ * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Soap_Client
@@ -80,6 +79,9 @@ class Zend_Soap_Client
     protected $_passphrase          = null;
     protected $_compression         = null;
     protected $_connection_timeout  = null;
+    protected $_stream_context      = null;
+    protected $_features            = null;
+    protected $_cache_wsdl          = null;
 
     /**
      * WSDL used to access server
@@ -103,6 +105,33 @@ class Zend_Soap_Client
      */
     protected $_lastMethod = '';
 
+    /**
+     * SOAP request headers.
+     *
+     * Array of SoapHeader objects
+     *
+     * @var array
+     */
+    protected $_soapInputHeaders = array();
+
+    /**
+     * Permanent SOAP request headers (shared between requests).
+     *
+     * Array of SoapHeader objects
+     *
+     * @var array
+     */
+    protected $_permanentSoapInputHeaders = array();
+
+    /**
+     * Output SOAP headers.
+     *
+     * Array of SoapHeader objects
+     *
+     * @var array
+     */
+    protected $_soapOutputHeaders = array();
+
     /**
      * Constructor
      *
@@ -112,6 +141,7 @@ class Zend_Soap_Client
     public function __construct($wsdl = null, $options = null)
     {
         if (!extension_loaded('soap')) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('SOAP extension is not loaded.');
         }
 
@@ -152,12 +182,16 @@ class Zend_Soap_Client
      *
      * Allows setting options as an associative array of option => value pairs.
      *
-     * @param  array $options
+     * @param  array|Zend_Config $options
      * @return Zend_Soap_Client
      * @throws Zend_SoapClient_Exception
      */
-    public function setOptions(array $options)
+    public function setOptions($options)
     {
+        if($options instanceof Zend_Config) {
+            $options = $options->toArray();
+        }
+
         foreach ($options as $key => $value) {
             switch ($key) {
                 case 'classmap':
@@ -213,6 +247,15 @@ class Zend_Soap_Client
                 case 'compression':
                     $this->setCompressionOptions($value);
                     break;
+                case 'stream_context':
+                    $this->setStreamContext($value);
+                    break;
+                case 'features':
+                    $this->setSoapFeatures($value);
+                    break;
+                case 'cache_wsdl':
+                    $this->setWsdlCache($value);
+                    break;
 
                 // Not used now
                 // case 'connection_timeout':
@@ -220,6 +263,7 @@ class Zend_Soap_Client
                 //    break;
 
                 default:
+                    require_once 'Zend/Soap/Client/Exception.php';
                     throw new Zend_Soap_Client_Exception('Unknown SOAP client option');
                     break;
             }
@@ -254,7 +298,10 @@ class Zend_Soap_Client
         $options['local_cert']     = $this->getHttpsCertificate();
         $options['passphrase']     = $this->getHttpsCertPassphrase();
         $options['compression']    = $this->getCompressionOptions();
-//        $options['connection_timeout'] = $this->_connection_timeout;
+        //$options['connection_timeout'] = $this->_connection_timeout;
+        $options['stream_context'] = $this->getStreamContext();
+        $options['cache_wsdl']     = $this->getWsdlCache();
+        $options['features']       = $this->getSoapFeatures();
 
         foreach ($options as $key => $value) {
             if ($value == null) {
@@ -275,6 +322,7 @@ class Zend_Soap_Client
     public function setSoapVersion($version)
     {
         if (!in_array($version, array(SOAP_1_1, SOAP_1_2))) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('Invalid soap version specified. Use SOAP_1_1 or SOAP_1_2 constants.');
         }
         $this->_soapVersion = $version;
@@ -305,6 +353,7 @@ class Zend_Soap_Client
     {
         foreach ($classmap as $type => $class) {
             if (!class_exists($class)) {
+                require_once 'Zend/Soap/Client/Exception.php';
                 throw new Zend_Soap_Client_Exception('Invalid class in class map');
             }
         }
@@ -336,6 +385,7 @@ class Zend_Soap_Client
     public function setEncoding($encoding)
     {
         if (!is_string($encoding)) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('Invalid encoding specified');
         }
 
@@ -370,6 +420,7 @@ class Zend_Soap_Client
             return true;
         }
 
+        require_once 'Zend/Soap/Client/Exception.php';
         throw new Zend_Soap_Client_Exception('Invalid URN');
     }
 
@@ -441,6 +492,7 @@ class Zend_Soap_Client
     public function setStyle($style)
     {
         if (!in_array($style, array(SOAP_RPC, SOAP_DOCUMENT))) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('Invalid request style specified. Use SOAP_RPC or SOAP_DOCUMENT constants.');
         }
 
@@ -471,6 +523,7 @@ class Zend_Soap_Client
     public function setEncodingMethod($use)
     {
         if (!in_array($use, array(SOAP_ENCODED, SOAP_LITERAL))) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('Invalid message encoding method. Use SOAP_ENCODED or SOAP_LITERAL constants.');
         }
 
@@ -641,6 +694,7 @@ class Zend_Soap_Client
     public function setHttpsCertificate($localCert)
     {
         if (!is_readable($localCert)) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('Invalid HTTPS client certificate path.');
         }
 
@@ -721,6 +775,81 @@ class Zend_Soap_Client
         return $this->_proxy_password;
     }
 
+    /**
+     * Set Stream Context
+     *
+     * @return Zend_Soap_Client
+     */
+    public function setStreamContext($context)
+    {
+        if(!is_resource($context) || get_resource_type($context) !== "stream-context") {
+            /**
+             * @see Zend_Soap_Client_Exception
+             */
+            require_once "Zend/Soap/Client/Exception.php";
+            throw new Zend_Soap_Client_Exception(
+                "Invalid stream context resource given."
+            );
+        }
+
+        $this->_stream_context = $context;
+        return $this;
+    }
+
+    /**
+     * Get Stream Context
+     *
+     * @return resource
+     */
+    public function getStreamContext()
+    {
+        return $this->_stream_context;
+    }
+
+    /**
+     * Set the SOAP Feature options.
+     *
+     * @param  string|int $feature
+     * @return Zend_Soap_Client
+     */
+    public function setSoapFeatures($feature)
+    {
+        $this->_features = $feature;
+
+        $this->_soapClient = null;
+        return $this;
+    }
+
+    /**
+     * Return current SOAP Features options
+     *
+     * @return int
+     */
+    public function getSoapFeatures()
+    {
+        return $this->_features;
+    }
+
+    /**
+     * Set the SOAP Wsdl Caching Options
+     *
+     * @param string|int|boolean $caching
+     * @return Zend_Soap_Client
+     */
+    public function setWsdlCache($options)
+    {
+        $this->_cache_wsdl = $options;
+        return $this;
+    }
+
+    /**
+     * Get current SOAP Wsdl Caching option
+     */
+    public function getWsdlCache()
+    {
+        return $this->_cache_wsdl;
+    }
+
     /**
      * Retrieve request XML
      *
@@ -764,7 +893,7 @@ class Zend_Soap_Client
     }
 
     /**
-     * Retrieve response headers
+     * Retrieve response headers (as string)
      *
      * @return string
      */
@@ -821,19 +950,22 @@ class Zend_Soap_Client
         $wsdl = $this->getWsdl();
         $options = array_merge($this->getOptions(), array('trace' => true));
 
-
         if ($wsdl == null) {
             if (!isset($options['location'])) {
+                require_once 'Zend/Soap/Client/Exception.php';
                 throw new Zend_Soap_Client_Exception('\'location\' parameter is required in non-WSDL mode.');
             }
             if (!isset($options['uri'])) {
+                require_once 'Zend/Soap/Client/Exception.php';
                 throw new Zend_Soap_Client_Exception('\'uri\' parameter is required in non-WSDL mode.');
             }
         } else {
             if (isset($options['use'])) {
+                require_once 'Zend/Soap/Client/Exception.php';
                 throw new Zend_Soap_Client_Exception('\'use\' parameter only works in non-WSDL mode.');
             }
             if (isset($options['style'])) {
+                require_once 'Zend/Soap/Client/Exception.php';
                 throw new Zend_Soap_Client_Exception('\'style\' parameter only works in non-WSDL mode.');
             }
         }
@@ -869,6 +1001,47 @@ class Zend_Soap_Client
         return $result;
     }
 
+    /**
+     * Add SOAP input header
+     *
+     * @param SoapHeader $header
+     * @param boolean $permanent
+     * @return Zend_Soap_Client
+     */
+    public function addSoapInputHeader(SoapHeader $header, $permanent = false)
+    {
+    	if ($permanent) {
+    		$this->_permanentSoapInputHeaders[] = $header;
+    	} else {
+    		$this->_soapInputHeaders[] = $header;
+    	}
+
+    	return $this;
+    }
+
+    /**
+     * Reset SOAP input headers
+     *
+     * @return Zend_Soap_Client
+     */
+    public function resetSoapInputHeaders()
+    {
+        $this->_permanentSoapInputHeaders = array();
+        $this->_soapInputHeaders = array();
+
+        return $this;
+    }
+
+    /**
+     * Get last SOAP output headers
+     *
+     * @return array
+     */
+    public function getLastSoapOutputHeaderObjects()
+    {
+    	return $this->_soapOutputHeaders;
+    }
+
     /**
      * Perform a SOAP call
      *
@@ -878,13 +1051,19 @@ class Zend_Soap_Client
      */
     public function __call($name, $arguments)
     {
-        if ($this->_soapClient == null) {
-            $this->_initSoapClientObject();
-        }
+        $soapClient = $this->getSoapClient();
 
-        $this->_lastMethod = $name; 
-        
-        $result = call_user_func_array(array($this->_soapClient, $name), $this->_preProcessArguments($arguments));
+        $this->_lastMethod = $name;
+
+        $soapHeaders = array_merge($this->_permanentSoapInputHeaders, $this->_soapInputHeaders);
+        $result = $soapClient->__soapCall($name,
+                                                 $this->_preProcessArguments($arguments),
+                                                 null, /* Options are already set to the SOAP client object */
+                                                 (count($soapHeaders) > 0)? $soapHeaders : null,
+                                                 $this->_soapOutputHeaders);
+
+        // Reset non-permanent input headers
+        $this->_soapInputHeaders = array();
 
         return $this->_preProcessResult($result);
     }
@@ -899,14 +1078,12 @@ class Zend_Soap_Client
     public function getFunctions()
     {
         if ($this->getWsdl() == null) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('\'getFunctions\' method is available only in WSDL mode.');
         }
 
-        if ($this->_soapClient == null) {
-            $this->_initSoapClientObject();
-        }
-
-        return $this->_soapClient->__getFunctions();
+        $soapClient = $this->getSoapClient();
+        return $soapClient->__getFunctions();
     }
 
 
@@ -925,13 +1102,45 @@ class Zend_Soap_Client
     public function getTypes()
     {
         if ($this->getWsdl() == null) {
+            require_once 'Zend/Soap/Client/Exception.php';
             throw new Zend_Soap_Client_Exception('\'getTypes\' method is available only in WSDL mode.');
         }
 
+        $soapClient = $this->getSoapClient();
+
+        return $soapClient->__getTypes();
+    }
+
+    /**
+     * @param SoapClient $soapClient
+     * @return Zend_Soap_Client
+     */
+    public function setSoapClient(SoapClient $soapClient)
+    {
+        $this->_soapClient = $soapClient;
+        return $this;
+    }
+
+    /**
+     * @return SoapClient
+     */
+    public function getSoapClient()
+    {
         if ($this->_soapClient == null) {
             $this->_initSoapClientObject();
         }
+        return $this->_soapClient;
+    }
 
-        return $this->_soapClient->__getTypes();
+    /**
+     * @param string $name
+     * @param string $value
+     * @return Zend_Soap_Client
+     */
+    public function setCookie($cookieName, $cookieValue=null)
+    {
+        $soapClient = $this->getSoapClient();
+        $soapClient->__setCookie($cookieName, $cookieValue);
+        return $this;
     }
 }
